from datetime import datetime
import json


# Application Configuration
class AppConfig:
    sampling_percentage = None
    last_updated_time = None
    config_dict = None


def get_config(api_client, config_dict, cached_config_etag):
    """Get Config"""
    sample_rate = 100
    try:
        config_api_response = api_client.get_app_config()
        response_config_etag = config_api_response.headers.get("X-Moesif-Config-ETag")
        if cached_config_etag:
            if cached_config_etag in config_dict: del config_dict[cached_config_etag]
        config_dict[response_config_etag] = json.loads(config_api_response.raw_body)
        app_config = config_dict.get(response_config_etag)
        if app_config is not None:
            sample_rate = app_config.get('sample_rate', 100)
        last_updated_time = datetime.utcnow()
    except:
        last_updated_time = datetime.utcnow()
    return last_updated_time, sample_rate, config_dict


def set_config(updated_time, sample_rate, config_dict):
    """Set global config"""
    AppConfig.last_updated_time = updated_time
    AppConfig.sampling_percentage = sample_rate
    AppConfig.config_dict = config_dict



"""
__author__ = 'mpetyx (Michael Petychakis)'
__version__ = "1.0.0"
__maintainer__ = "Michael Petychakis"
__email__ = "michael@orfium.com"
__status__ = "Production"
"""

from __future__ import absolute_import, unicode_literals
from django.conf import settings
from .http_response_catcher import HttpResponseCatcher
from celery import shared_task
from moesifapi.moesif_api_client import MoesifAPIClient
from moesifapi.models import EventModel
from .app_config import AppConfig, get_config, set_config
from datetime import datetime, timedelta

middleware_settings = settings.MOESIF_MIDDLEWARE
client = MoesifAPIClient(middleware_settings.get('APPLICATION_ID'))
BATCH_SIZE = settings.MOESIF_MIDDLEWARE.get('BATCH_SIZE', 25)
DEBUG = middleware_settings.get('LOCAL_DEBUG', False)

api_client = client.api
response_catcher = HttpResponseCatcher()
api_client.http_call_back = response_catcher

try:
    get_broker_url = settings.BROKER_URL
    if get_broker_url:
        BROKER_URL = get_broker_url
    else:
        BROKER_URL = None
except AttributeError:
    BROKER_URL = settings.MOESIF_MIDDLEWARE.get('CELERY_BROKER_URL', None)

def queue_get_all(queue_events):
    events = []
    max_events_to_retreive = BATCH_SIZE
    for num_of_events_retrieved in range(0, max_events_to_retreive):
        try:
            if num_of_events_retrieved == max_events_to_retreive:
                break
            events.append(queue_events.get_nowait())
        except:
            break
    return events

@shared_task(ignore_result=True)
def async_client_create_event():
    batch_events = []
    try:
        with Connection(BROKER_URL) as conn:
            moesif_events_queue = conn.SimpleQueue('moesif_events_queue')
            try:
                queue_size = moesif_events_queue.qsize()
                while queue_size > 0:
                    message = queue_get_all(moesif_events_queue)
                    for event in message:
                        batch_events.append(EventModel().from_dictionary(event.payload))
                        event.ack()
                    try:
                        queue_size = moesif_events_queue.qsize()
                    except ChannelError:
                        queue_size = 0
                    moesif_events_queue.close()
            except ChannelError:
                if DEBUG:
                    print("No message to read from the queue")
    except:
        if DEBUG:
            print("Error while connecting to - {0}".format(BROKER_URL))

    if batch_events:
        if DEBUG:
            print("Sending events to Moesif")
        batch_events_api_response = api_client.create_events_batch(batch_events)
        cached_config_etag = next(iter(AppConfig.config_dict))
        batch_events_response_config_etag = batch_events_api_response.get("X-Moesif-Config-ETag")
        if batch_events_response_config_etag is not None \
                and cached_config_etag != batch_events_response_config_etag \
                and datetime.utcnow() > AppConfig.last_updated_time + timedelta(minutes=5):

            AppConfig.last_updated_time, AppConfig.sampling_percentage, AppConfig.config_dict = get_config(api_client, AppConfig.config_dict,
                                                                                    cached_config_etag)
            set_config(AppConfig.last_updated_time, AppConfig.sampling_percentage, AppConfig.config_dict)
        if DEBUG:
            print("Events sent successfully")
    else:
        if DEBUG:
            print("No events to send")


CELERY = False
if settings.MOESIF_MIDDLEWARE.get('USE_CELERY', False):
    if BROKER_URL:
        try:
            from apscheduler.schedulers.background import BackgroundScheduler
            from apscheduler.triggers.interval import IntervalTrigger
            import atexit
            from kombu import Connection
            from kombu.exceptions import ChannelError

            scheduler = BackgroundScheduler(daemon=True)
            scheduler.start()
            scheduler.add_job(
                func=async_client_create_event,
                trigger=IntervalTrigger(seconds=5),
                id='moesif_events_batch_job',
                name='Schedule events batch job every 5 second',
                replace_existing=True)

            # Shut down the scheduler when exiting the app
            atexit.register(lambda: scheduler.shutdown())
        except:
            if DEBUG:
                print("Error when scheduling the job")
    else:
        if DEBUG:
            print("Unable to schedule the job as the BROKER_URL is not provided")



# -*- coding: utf-8 -*-

"""
    setup module.
    See:
        https://packaging.python.org/en/latest/distributing.html
        https://github.com/pypa/sampleproject

"""

# Always prefer setuptools over distutils
from setuptools import setup, find_packages
# To use a consistent encoding
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
long_description = ''
if path.exists('README.rst'):
    with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
        long_description = f.read()

setup(
    name='moesifdjango',

    # Versions should comply with PEP440.  For a discussion on single-sourcing
    # the version across setup.py and the project code, see
    # https://packaging.python.org/en/latest/single_source_version.html
    version='1.5.3',

    description='Moesif Middleware for Python Django',
    long_description=long_description,

    # The project's main homepage.
    url='https://www.moesif.com/docs/server-integration/django/',

    # Author details
    author='Moesif, Inc',
    author_email='derric@moesif.com',

    license='Apache Software License',

    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 4 - Beta',

        # Indicate who your project is intended for
        'Intended Audience :: Developers',
        'Topic :: Internet :: Log Analysis',
        'Topic :: Software Development :: Debuggers',
        'Topic :: Software Development :: Libraries',
        'Topic :: Internet :: WWW/HTTP',

        # Pick your license as you wish (should match "license" above)
        'License :: OSI Approved :: Apache Software License',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],

    keywords='log analysis restful api development debug',

    # You can just specify the packages manually here if your project is
    # simple. Or you can use find_packages().
    packages=find_packages(exclude=['contrib', 'docs', 'tests']),

    # List run-time dependencies here.  These will be installed by pip when
    # your project is installed. For an analysis of "install_requires" vs pip's
    # requirements files see:
    # https://packaging.python.org/en/latest/requirements.html
    install_requires=['requests', 'jsonpickle', 'python-dateutil', 'isodatetimehandler', 'moesifapi', 'moesifpythonrequest'],

    # List additional groups of dependencies here (e.g. development
    # dependencies). You can install these using the following syntax,
    # for example:
    # $ pip install -e .[dev,test]
    extras_require={
        'dev': [],
        'test': ['nose'],
    },

)
